#include "window.h"
#include "ui_utils.h"
#ifdef MACOSX
#include "sys/osx/dark_theme.h"
#else
#include "sys/win32/dark_theme.h"
#endif

QIcon UiUtils::CreateSideBarIcon(const char* file) {
	QPixmap pixmap(file, "PNG");
	QIcon result;
	result.addPixmap(pixmap, QIcon::Normal);
	result.addPixmap(pixmap, QIcon::Selected);
	return result;
}

bool UiUtils::IsInDarkMode() {
	if (session.config.theme != OS)
		return (session.config.theme == DARK);
#ifdef MACOSX
	if (osx::DarkThemeAvailable()) {
		if (osx::IsInDarkTheme()) {
			return true;
		} else {
			return false;
		}
	}
#elif defined(WIN32)
	if (win32::DarkThemeAvailable()) {
		if (win32::IsInDarkTheme()) {
			return true;
		} else {
			return false;
		}
	}
#endif
	return (session.config.theme == DARK);
}

void UiUtils::CreateTextHeader(QWidget* parent, QString title, QPoint point, QSize size) {
	QLabel* static_text_title  = new QLabel(title, parent);
	static_text_title->setTextFormat(Qt::PlainText);
	static_text_title->setStyleSheet("font-weight: bold");
	static_text_title->move(point.x(), point.y());
	static_text_title->resize(size.width(), 16);

	QFrame* static_text_line = new QFrame(parent);
	static_text_line->setFrameShape(QFrame::HLine);
	static_text_line->setFrameShadow(QFrame::Sunken);
	static_text_line->resize(size.width(), 2);
	static_text_line->move(point.x(), point.y()+18);
}

QPlainTextEdit* UiUtils::CreateTextParagraph(QWidget* parent, QString title, QString data, QPoint point, QSize size) {
	CreateTextHeader(parent, title, point, size);

	QPlainTextEdit* paragraph = new QPlainTextEdit(data, parent);
	paragraph->setTextInteractionFlags(Qt::NoTextInteraction);
	paragraph->setAttribute(Qt::WidgetAttribute::WA_TransparentForMouseEvents);
	paragraph->setWordWrapMode(QTextOption::NoWrap);
	paragraph->setFrameShape(QFrame::NoFrame);
	paragraph->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	paragraph->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	paragraph->setStyleSheet("background: transparent;");
	paragraph->move(point.x()+12, point.y()+32);
	paragraph->resize(size.width()-12, size.height()-32);
	return paragraph;
}

QPlainTextEdit* UiUtils::CreateTextParagraphWithLabels(QWidget* parent, QString title, QString label, QString data, QPoint point, QSize size) {
	CreateTextHeader(parent, title, point, size);

	QPlainTextEdit* label_t = new QPlainTextEdit(label, parent);
	label_t->setTextInteractionFlags(Qt::NoTextInteraction);
	label_t->setAttribute(Qt::WidgetAttribute::WA_TransparentForMouseEvents);
	label_t->setWordWrapMode(QTextOption::NoWrap);
	label_t->setFrameShape(QFrame::NoFrame);
	label_t->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	label_t->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	label_t->setStyleSheet("background: transparent;");
	label_t->move(point.x()+12, point.y()+32);
	label_t->resize(90, size.height()-32);

	QPlainTextEdit* paragraph = new QPlainTextEdit(data, parent);
	paragraph->setTextInteractionFlags(Qt::NoTextInteraction);
	paragraph->setAttribute(Qt::WidgetAttribute::WA_TransparentForMouseEvents);
	paragraph->setWordWrapMode(QTextOption::NoWrap);
	paragraph->setFrameShape(QFrame::NoFrame);
	paragraph->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	paragraph->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	paragraph->setStyleSheet("background: transparent;");
	paragraph->move(point.x()+102, point.y()+32);
	paragraph->resize(size.width()-102, size.height()-32);
	return paragraph;
}

/* As far as I can tell, this is identical to the way Taiga implements it.
   Kind of cool, I didn't even look into the source code for it :p */
QPlainTextEdit* UiUtils::CreateSelectableTextParagraph(QWidget* parent, QString title, QString data, QPoint point, QSize size) {
	CreateTextHeader(parent, title, point, size);

	QPlainTextEdit* text_edit = new QPlainTextEdit(data, parent);
	text_edit->setReadOnly(true);
	text_edit->setFrameShape(QFrame::NoFrame);
	text_edit->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	text_edit->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	text_edit->setStyleSheet("background: transparent;");
	text_edit->move(point.x()+12, point.y()+32);
	text_edit->resize(size.width()-12, size.height()-32);
	return text_edit;
}
